@@ -4,7 +4,7 @@ from django.contrib import messages
 from django.db.models import Q
 from django.http import HttpResponseRedirect, Http404, JsonResponse
 from django.views.generic import FormView, RedirectView, TemplateView, View
-from django.utils.translation import gettext_lazy as _
+from django.utils.translation import gettext, gettext_lazy as _
 from django.utils import timezone
 from django.utils.dateparse import parse_datetime
 
@@ -15,10 +15,13 @@ import json
 from braces.views import PermissionRequiredMixin
 
 from .models import (
-    Event, Series, PublicEvent, Invoice, InvoiceItem,
+    Event, Series, PublicEvent, Invoice, InvoiceItem, Customer,
     CashPaymentRecord, DanceRole, Registration, EventRegistration
 )
-from .forms import ClassChoiceForm, RegistrationContactForm
+from .forms import (
+    ClassChoiceForm, RegistrationContactForm, MultiRegCustomerNameForm,
+    PartnerRequiredForm
+)
 from .constants import getConstant, REG_VALIDATION_STR
 from .signals import (
     post_student_info, apply_discount, apply_price_adjustments,
@@ -56,9 +59,7 @@ class ClassRegistrationView(FinancialContextMixin, EventOrderMixin, SiteHistoryM
     form_class = ClassChoiceForm
     template_name = 'core/registration/event_registration.html'
     returnJson = False
-    includeCounts = True
     voucherField = False
-    pluralName = True
 
     # The temporary registration and the list of event registrations is kept
     # as an attribute of the view so that it may be used in subclassed versions
@@ -113,7 +114,7 @@ class ClassRegistrationView(FinancialContextMixin, EventOrderMixin, SiteHistoryM
         })
 
         # Automatically pass along some of the optional form kwargs
-        for key in ['includeCounts', 'pluralName', 'voucherField']:
+        for key in ['voucherField', ]:
             if isinstance(getattr(self, key, None), bool):
                 kwargs[key] = getattr(self, key, None)
 
@@ -141,27 +142,44 @@ class ClassRegistrationView(FinancialContextMixin, EventOrderMixin, SiteHistoryM
 
         # The session expires after a period of inactivity that is specified in preferences.
         expiry = timezone.now() + timedelta(minutes=getConstant('registration__sessionExpiryMinutes'))
-        permitted_keys = getattr(form, 'permitted_event_keys', ['role', ])
 
         # These are prefixes that may be used for events in the form.  Each one
         # corresponds to a polymorphic content type (PublicEvent, Series, etc.)
-        event_types = ['event_', 'publicevent_', 'series_']
+        event_types = ['event', 'publicevent', 'series', 'privatelessonevent']
 
         try:
             event_listing = {}
             for key, value in form.cleaned_data.items():
-                if any(x in key for x in event_types) and value:
-                    this_event = {}
-                    for y in value:
-                        for k,v in json.loads(y).items():
-                            if k in permitted_keys and k not in this_event:
-                                this_event[k] = v
-                    event_listing[int(key.split("_")[-1])] = this_event
+                if key.split('_')[0] in event_types and value:
+                    # There may be more than one registration per event, but we
+                    # also want only one drop-in registration if possible, to
+                    # avoid issues with automatic event check-ins.
+                    this_event_list = []
+                    this_dropin = {'dropIn': True, 'occurrences': []}
+                    for y in [x for x in value if x[1] != 0]:
+                        this_quantity = y[1]
+
+                        if y[0].startswith('dropin_') and this_quantity == 1:
+                            this_dropin['occurrences'].append(int(y[0].split('_')[-1]))
+                        else:
+                            this_event_item = {'role': None}
+                            if y[0].startswith('role_'):
+                                this_event_item = {'role': int(y[0].split('_')[-1])}
+                            elif y[0].startswith('dropin_'):
+                                this_event_item.update({
+                                    'dropIn': True,
+                                    'occurrences': [int(y[0].split('_')[-1]),]
+                                })
+                            for i in range(this_quantity):
+                                this_event_list.append(this_event_item)
+                    if this_dropin.get('occurrences', []):
+                        this_event_list.append(this_dropin)
+                    event_listing[int(key.split("_")[-1])] = this_event_list
             non_event_listing = {
                 key: value for key, value in form.cleaned_data.items() if
-                not any(x in key for x in event_types)
+                not key.split('_')[0] in event_types
             }
-        except (ValueError, TypeError) as e:
+        except (ValueError, TypeError, IndexError) as e:
             form.add_error(None, ValidationError(_('Invalid event information passed.'), code='invalid'))
             return self.form_invalid(form)
 
@@ -189,80 +207,81 @@ class ClassRegistrationView(FinancialContextMixin, EventOrderMixin, SiteHistoryM
         # from the form submission data.
         self.event_registrations = []
 
-        for key, value in event_listing.items():
+        for key, eventRegs in event_listing.items():
             this_event = associated_events.get(id=key)
 
-            # Check if registration is still feasible based on both completed registrations
-            # and registrations that are not yet complete
-            this_role_id = value.get('role', None) if 'role' in permitted_keys else None
-            soldOut = this_event.soldOutForRole(role=this_role_id, includeTemporaryRegs=True)
+            for value in eventRegs:
+                # Check if registration is still feasible based on both completed registrations
+                # and registrations that are not yet complete
+                this_role_id = value.get('role', None)
+                soldOut = this_event.soldOutForRole(role=this_role_id, includeTemporaryRegs=True)
 
-            if soldOut:
-                if self.request.user.has_perm('core.override_register_soldout'):
-                    # This message will be displayed on the Step 2 page by default.
-                    messages.warning(self.request, _(
-                        'Registration for \'%s\' is sold out. ' % this_event.name +
-                        'Based on your user permission level, you may proceed ' +
-                        'with registration.  However, if you do not wish to exceed ' +
-                        'the listed capacity of the event, please do not proceed.'
-                    ))
-                else:
-                    # For users without permissions, don't allow registration for sold out things
-                    # at all.
-                    form.add_error(None, ValidationError(
-                        _(
-                            'Registration for "%s" is tentatively sold out while ' +
-                            'others complete their registration.  Please try ' +
-                            'again later.' % this_event.name
-                        ), code='invalid')
+                if soldOut:
+                    if self.request.user.has_perm('core.override_register_soldout'):
+                        # This message will be displayed on the Step 2 page by default.
+                        messages.warning(self.request, _(
+                            'Registration for \'%s\' is sold out. ' % this_event.name +
+                            'Based on your user permission level, you may proceed ' +
+                            'with registration.  However, if you do not wish to exceed ' +
+                            'the listed capacity of the event, please do not proceed.'
+                        ))
+                    else:
+                        # For users without permissions, don't allow registration for sold out things
+                        # at all.
+                        form.add_error(None, ValidationError(
+                            _(
+                                'Registration for "%s" is tentatively sold out while ' +
+                                'others complete their registration.  Please try ' +
+                                'again later.' % this_event.name
+                            ), code='invalid')
+                        )
+                        return self.form_invalid(form)
+
+                dropInList = value.get('occurrences', []) if value.get('dropIn', False) else []
+
+                # If nothing is sold out, then proceed to create Registration and
+                # EventRegistration objects for the items selected by this form.  The
+                # expiration date is set to be identical to that of the session.
+
+                logger.debug('Creating temporary event registration for: %s' % key)
+                if len(dropInList) > 0:
+                    this_price = this_event.getBasePrice(dropIns=len(dropInList))
+                    tr = EventRegistration(
+                        event=this_event, dropIn=True,
                     )
-                    return self.form_invalid(form)
+                else:
+                    this_price = this_event.getBasePrice(payAtDoor=reg.payAtDoor)
+                    tr = EventRegistration(
+                        event=this_event, role_id=this_role_id
+                    )
+                # If it's possible to store additional data and such data exist, then store them.
+                tr.data = {k: v for k, v in value.items() if k not in ['role', 'dropIn', 'occurrences']}
+                if dropInList:
+                    tr.data['__dropInOccurrences'] = dropInList
 
-            dropInList = [int(k.split("_")[-1]) for k, v in value.items() if k.startswith('dropin_') and v is True]
+                checkin_rule = getConstant('registration__doorCheckInRule')
+                if reg.payAtDoor and checkin_rule == 'E':
+                    # Check into the full event
+                    tr.data['__checkInEvent'] = True
+                elif reg.payAtDoor and checkin_rule == 'O' and dropInList:
+                    # Check into the first upcoming drop-in occurrence
+                    best_occ = tr.event.eventoccurrence_set.filter(
+                        id__in=dropInList,
+                        startTime__gte=ensure_localtime(timezone.now()) - timedelta(minutes=45)
+                    ).first()
+                    tr.data['__checkInOccurrence'] = getattr(best_occ, 'id', None)
+                elif reg.payAtDoor and checkin_rule == 'O':
+                    # Check into the next upcoming occurrence (45 min. grace period)
+                    tr.data['__checkInOccurrence'] = getattr(
+                        tr.event.getNextOccurrence(
+                            ensure_localtime(timezone.now()) - timedelta(minutes=45)
+                        ),
+                        'id',
+                        None
+                    )
 
-            # If nothing is sold out, then proceed to create Registration and
-            # EventRegistration objects for the items selected by this form.  The
-            # expiration date is set to be identical to that of the session.
-
-            logger.debug('Creating temporary event registration for: %s' % key)
-            if len(dropInList) > 0:
-                this_price = this_event.getBasePrice(dropIns=len(dropInList))
-                tr = EventRegistration(
-                    event=this_event, dropIn=True,
-                )
-            else:
-                this_price = this_event.getBasePrice(payAtDoor=reg.payAtDoor)
-                tr = EventRegistration(
-                    event=this_event, role_id=this_role_id
-                )
-            # If it's possible to store additional data and such data exist, then store them.
-            tr.data = {k: v for k, v in value.items() if k in permitted_keys and k != 'role'}
-            if dropInList:
-                tr.data['__dropInOccurrences'] = dropInList
-
-            checkin_rule = getConstant('registration__doorCheckInRule')
-            if reg.payAtDoor and checkin_rule == 'E':
-                # Check into the full event
-                tr.data['__checkInEvent'] = True
-            elif reg.payAtDoor and checkin_rule == 'O' and dropInList:
-                # Check into the first upcoming drop-in occurrence
-                best_occ = tr.event.eventoccurrence_set.filter(
-                    id__in=dropInList,
-                    startTime__gte=ensure_localtime(timezone.now()) - timedelta(minutes=45)
-                ).first()
-                tr.data['__checkInOccurrence'] = getattr(best_occ, 'id', None)
-            elif reg.payAtDoor and checkin_rule == 'O':
-                # Check into the next upcoming occurrence (45 min. grace period)
-                tr.data['__checkInOccurrence'] = getattr(
-                    tr.event.getNextOccurrence(
-                        ensure_localtime(timezone.now()) - timedelta(minutes=45)
-                    ),
-                    'id',
-                    None
-                )
-
-            tr.data['__price'] = this_price
-            self.event_registrations.append(tr)
+                tr.data['__price'] = this_price
+                self.event_registrations.append(tr)
 
         # If we got this far with no issues, then save
         invoice = reg.link_invoice(expirationDate=expiry)
@@ -621,7 +640,10 @@ class AjaxClassRegistrationView(PermissionRequiredMixin, RegistrationAdjustments
                     prior_response=response, request=request
                 )
                 if signal_response.get('status') != 'success':
-                    errors += signal_response.get('errors', [])
+                    # Add the errors but avoid duplicates from things like
+                    # duplicate event registrations.
+                    for error in signal_response.get('errors', []):
+                        errors.append(error) if error not in errors else errors
                 this_item_response.update(signal_response.get('response',{}))
 
             signal_responses = get_invoice_item_related.send(
@@ -834,8 +856,8 @@ class AjaxClassRegistrationView(PermissionRequiredMixin, RegistrationAdjustments
         '''
 
         # Used for specifying defaults
-        reg_nullable_keys = ['firstName', 'lastName', 'email', 'phone', 'howHeardAboutUs']
-        reg_bool_keys = ['student', 'payAtDoor']
+        reg_nullable_keys = ['howHeardAboutUs',]
+        reg_bool_keys = ['payAtDoor',]
 
         eventreg_items = [x for x in post_data.get('items', []) if x.get('type', None) == 'eventRegistration']
         if not eventreg_items:
@@ -963,30 +985,70 @@ class AjaxClassRegistrationView(PermissionRequiredMixin, RegistrationAdjustments
                 )
             )
         )):
-            errors.append({
-                'code': 'duplicate_event',
-                'message': _(
-                    'You cannot register more than once for event: {event_name}. '.format(
-                        event_name=this_event.name
-                    ) +
-                    'Please remove the existing item from your cart and try again.'
-                )
-            })
+            this_code = 'duplicate_event_%s' % this_event.id
+
+            if getConstant('registration__multiRegNameFormRule') == 'Y':
+
+                # Add a messages unless one already exists.
+                storage = messages.get_messages(request)
+                existing = [x for x in storage if this_code in x.extra_tags]
+                storage.used = False
+
+                if not existing:
+                    messages.info(
+                        request, gettext(
+                            'Note: You cannot register more than once under the same name ' +
+                            'for event {event_name}. '.format(event_name=this_event.name) +
+                            'Please ensure that you enter the correct names for ' +
+                            'each registration in the next step.'
+                        ),
+                        extra_tags=this_code
+                    )
+            else:
+                errors.append({
+                    'code': this_code,
+                    'message': _(
+                        'You cannot register more than once for event: {event_name}. '.format(
+                            event_name=this_event.name
+                        ) +
+                        'Please remove the existing item from your cart and try again.'
+                    )
+                })
 
         elif (
             len(same_event) > 1 and
             len([x for x in same_event if x.get('dropIn', False) != response['dropIn']]) > 0
         ):
-            errors.append({
-                'code': 'dropin_with_full',
-                'message': _(
-                    'You cannot register as both a drop-in and as a ' +
-                    'full registrant for event: {event_name}. '.format(
-                        event_name=this_event.name
-                    ) +
-                    'Please remove the existing item from your cart and try again.'
-                )
-            })
+            this_code = 'dropin_with_full_%s' % this_event.id
+
+            if getConstant('registration__multiRegNameFormRule') == 'Y':
+
+                # Add a messages unless one already exists.
+                storage = messages.get_messages(request)
+                existing = [x for x in storage if this_code in x.extra_tags]
+                storage.used = False
+
+                if not existing:
+                    messages.info(
+                        request, gettext(
+                            'Note: You cannot register as both a drop-in and as a ' +
+                            'full registrant for event {event_name}. '.format(event_name=this_event.name) +
+                            'Please ensure that you enter the correct names for ' +
+                            'each registration in the next step.'
+                        ),
+                        extra_tags=this_code
+                    )
+            else:
+                errors.append({
+                    'code': this_code,
+                    'message': _(
+                        'You cannot register as both a drop-in and as a ' +
+                        'full registrant for event: {event_name}. '.format(
+                            event_name=this_event.name
+                        ) +
+                        'Please remove the existing item from your cart and try again.'
+                    )
+                })
 
         if not (
             this_event.registrationOpen or
@@ -1064,6 +1126,9 @@ class AjaxClassRegistrationView(PermissionRequiredMixin, RegistrationAdjustments
             item.grossTotal = this_event.getBasePrice(payAtDoor=registration.payAtDoor)
             this_eventreg.role = this_role
 
+        if post_data.get('student', False):
+            this_eventreg.student = post_data.get('student')
+
         if item_data.get('checkInType') in ['E', 'S'] and item_data.get('checkInOccurrence'):
             this_eventreg.data['__checkInOccurrence'] = item_data['checkInOccurrence']
         elif item_data.get('checkInType') == 'F':
@@ -1149,6 +1214,9 @@ class RegistrationSummaryView(
 ):
     template_name = 'core/registration_summary.html'
 
+    # Ensures that customer-specific discounts are applied at this stage
+    customers_final = True
+
     def dispatch(self, request, *args, **kwargs):
         ''' Always check that the temporary registration has not expired '''
         regSession = self.request.session.get(REG_VALIDATION_STR, {})
@@ -1327,6 +1395,302 @@ class RegistrationSummaryView(
         return context_data
 
 
+class PartnerRequiredView(RegistrationAdjustmentsMixin, FormView):
+    '''
+    When one or more events in a customer's registration have a partner
+    required, this page is used to collect partner name information for each
+    registrant to that event.
+    '''
+    form_class = PartnerRequiredForm
+    template_name = 'core/partner_required_form.html'
+
+    # Ensures that customer-specific discounts are applied at this stage
+    customers_final = True
+
+    def dispatch(self, request, *args, **kwargs):
+        '''
+        Require session data to be set to proceed, otherwise go back to step 1.
+        Because they have the same expiration date, this also implies that the
+        Registration object is not yet expired.
+        '''
+        if REG_VALIDATION_STR not in request.session:
+            return HttpResponseRedirect(reverse('registration'))
+
+        try:
+            self.invoice = Invoice.objects.get(
+                id=self.request.session[REG_VALIDATION_STR].get('invoiceId')
+            )
+        except ObjectDoesNotExist:
+            messages.error(request, _('Invalid invoice identifier passed to sign-up form.'))
+            return HttpResponseRedirect(reverse('registration'))
+
+        expiry = parse_datetime(
+            self.request.session[REG_VALIDATION_STR].get('invoiceExpiry', ''),
+        )
+        if not expiry or expiry < timezone.now():
+            messages.info(request, _('Your registration session has expired. Please try again.'))
+            return HttpResponseRedirect(reverse('registration'))
+
+        self.registration = Registration.objects.filter(
+            invoice=self.invoice
+        ).prefetch_related(
+            'eventregistration_set', 'eventregistration_set__event',
+            'eventregistration_set__customer'
+        ).first()
+
+        if not self.registration:
+            messages.error(request, _('Invalid registration passed to additional customer name form.'))
+            return HttpResponseRedirect(reverse('registration'))
+
+        self.partnerRequiredRegs = self.registration.eventregistration_set.filter(
+            event__partnerRequired=True
+        )
+
+        if not self.partnerRequiredRegs:
+            return HttpResponseRedirect(self.get_success_url())
+
+        return super().dispatch(request, *args, **kwargs)
+
+    def get_context_data(self, **kwargs):
+        context_data = super().get_context_data(**kwargs)
+        reg = self.registration
+
+        payAtDoor = self.request.session[REG_VALIDATION_STR].get('payAtDoor', False)
+
+        discount_codes, total_discount_amount = self.getDiscounts(
+            self.invoice, registration=reg
+        )
+        addons = self.getAddons(self.invoice, reg)
+
+        # Update totals (without saving anything), so that taxes are
+        # recalculated and the invoice handler knows how much to apply.
+        items_queryset = self.invoice.updateTotals(
+            save=False, allocateAmounts={'total': -1*total_discount_amount}
+        )
+
+        # Get a voucher ID to check from the current contents of the form
+        voucherId = self.invoice.data.get('gift', None)
+
+        if voucherId:
+            context_data['voucher'] = self.getVoucher(voucherId, self.invoice)
+
+            # Recalculate taxes, but do not save the invoice
+            # with the updates, since the voucher will only be applied later.
+            if context_data['voucher'].get('beforeTax'):
+                allocateAmounts = {'total': -1*context_data['voucher'].get('voucherAmount', 0)}
+            else:
+                allocateAmounts = {'adjustments': -1*context_data['voucher'].get('voucherAmount', 0)}
+            items_queryset = self.invoice.updateTotals(
+                save=False, allocateAmounts=allocateAmounts,
+                prior_queryset=items_queryset
+            )
+
+        context_data.update({
+            'invoice': self.invoice,
+            'currencySymbol': getConstant('general__currencySymbol'),
+            'reg': reg,
+            'payAtDoor': payAtDoor,
+            'addonItems': addons,
+            'discount_codes': discount_codes,
+            'discount_code_amount': total_discount_amount,
+        })
+
+        return context_data
+
+    def get_form_kwargs(self, **kwargs):
+        ''' Pass along the request data to the form '''
+        kwargs = super().get_form_kwargs(**kwargs)
+        kwargs['partnerRequiredRegs'] = self.partnerRequiredRegs
+        return kwargs
+
+    def get_success_url(self):
+        return reverse('showRegSummary')
+
+    def form_valid(self, form):
+        '''
+        Even if this form is valid, the handlers for this form may have added messages
+        to the request.  In that case, then the page should be handled as if the form
+        were invalid.  Otherwise, update the session data with the form data and then
+        move to the next view
+        '''
+
+        # The session expires after a period of inactivity that is specified in preferences.
+        expiry = timezone.now() + timedelta(minutes=getConstant('registration__sessionExpiryMinutes'))
+        self.request.session[REG_VALIDATION_STR]["invoiceExpiry"] = \
+            expiry.strftime('%Y-%m-%dT%H:%M:%S%z')
+        self.request.session.modified = True
+
+        for er in self.partnerRequiredRegs:
+            partner_data = {
+                'firstName': form.cleaned_data.pop('er_%s_partner_firstName' % er.id),
+                'lastName': form.cleaned_data.pop('er_%s_partner_lastName' % er.id)
+            }
+
+            customerId = form.cleaned_data.pop('er_%s_partner_customerId' % er.id, None)
+            if not Customer.objects.filter(id=customerId).exists():
+                customerId = None
+
+            if customerId:
+                partner_data['customerId'] = customerId
+
+            er.data['partner'] = partner_data
+            er.save()
+
+        return HttpResponseRedirect(self.get_success_url())  # Redirect after POST
+
+
+class MultiRegCustomerNameView(RegistrationAdjustmentsMixin, FormView):
+    '''
+    This page collects additional name and email information needed when there
+    are multiple EventRegistrations associated with an Invoice.  For each 
+
+    '''
+    form_class = MultiRegCustomerNameForm
+    template_name = 'core/multireg_customer_name_form.html'
+
+    def dispatch(self, request, *args, **kwargs):
+        '''
+        Require session data to be set to proceed, otherwise go back to step 1.
+        Because they have the same expiration date, this also implies that the
+        Registration object is not yet expired.
+        '''
+        if REG_VALIDATION_STR not in request.session:
+            return HttpResponseRedirect(reverse('registration'))
+
+        try:
+            self.invoice = Invoice.objects.get(
+                id=self.request.session[REG_VALIDATION_STR].get('invoiceId')
+            )
+        except ObjectDoesNotExist:
+            messages.error(request, _('Invalid invoice identifier passed to sign-up form.'))
+            return HttpResponseRedirect(reverse('registration'))
+
+        expiry = parse_datetime(
+            self.request.session[REG_VALIDATION_STR].get('invoiceExpiry', ''),
+        )
+        if not expiry or expiry < timezone.now():
+            messages.info(request, _('Your registration session has expired. Please try again.'))
+            return HttpResponseRedirect(reverse('registration'))
+
+        self.registration = Registration.objects.filter(
+            invoice=self.invoice
+        ).prefetch_related('eventregistration_set').first()
+
+        self.multiReg = (
+            self.registration and
+            self.registration.eventregistration_set.count() > 1
+        )
+
+        if not self.registration or not self.multiReg:
+            messages.error(request, _('Invalid registration passed to additional customer name form.'))
+            return HttpResponseRedirect(reverse('registration'))
+
+        self.partnerRequired = self.registration.eventregistration_set.filter(
+            event__partnerRequired=True
+        ).exists()
+
+        return super().dispatch(request, *args, **kwargs)
+
+    def get_context_data(self, **kwargs):
+        context_data = super().get_context_data(**kwargs)
+        reg = self.registration
+
+        payAtDoor = self.request.session[REG_VALIDATION_STR].get('payAtDoor', False)
+
+        discount_codes, total_discount_amount = self.getDiscounts(
+            self.invoice, registration=reg
+        )
+        addons = self.getAddons(self.invoice, reg)
+
+        # Update totals (without saving anything), so that taxes are
+        # recalculated and the invoice handler knows how much to apply.
+        items_queryset = self.invoice.updateTotals(
+            save=False, allocateAmounts={'total': -1*total_discount_amount}
+        )
+
+        # Get a voucher ID to check from the current contents of the form
+        voucherId = self.invoice.data.get('gift', None)
+
+        if voucherId:
+            context_data['voucher'] = self.getVoucher(voucherId, self.invoice)
+
+            # Recalculate taxes, but do not save the invoice
+            # with the updates, since the voucher will only be applied later.
+            if context_data['voucher'].get('beforeTax'):
+                allocateAmounts = {'total': -1*context_data['voucher'].get('voucherAmount', 0)}
+            else:
+                allocateAmounts = {'adjustments': -1*context_data['voucher'].get('voucherAmount', 0)}
+            items_queryset = self.invoice.updateTotals(
+                save=False, allocateAmounts=allocateAmounts,
+                prior_queryset=items_queryset
+            )
+
+        context_data.update({
+            'invoice': self.invoice,
+            'currencySymbol': getConstant('general__currencySymbol'),
+            'reg': reg,
+            'payAtDoor': payAtDoor,
+            'addonItems': addons,
+            'discount_codes': discount_codes,
+            'discount_code_amount': total_discount_amount,
+        })
+
+        return context_data
+
+    def get_form_kwargs(self, **kwargs):
+        ''' Pass along the request data to the form '''
+        kwargs = super().get_form_kwargs(**kwargs)
+        kwargs['request'] = self.request
+        kwargs['registration'] = self.registration
+        kwargs['invoice'] = self.invoice
+        kwargs['multiReg'] = self.multiReg
+        return kwargs
+
+    def get_success_url(self):
+        if self.partnerRequired:
+            return reverse('partnerRequiredForm')
+        return reverse('showRegSummary')
+
+    def form_valid(self, form):
+        '''
+        Even if this form is valid, the handlers for this form may have added messages
+        to the request.  In that case, then the page should be handled as if the form
+        were invalid.  Otherwise, update the session data with the form data and then
+        move to the next view
+        '''
+
+        # The session expires after a period of inactivity that is specified in preferences.
+        expiry = timezone.now() + timedelta(minutes=getConstant('registration__sessionExpiryMinutes'))
+        self.request.session[REG_VALIDATION_STR]["invoiceExpiry"] = \
+            expiry.strftime('%Y-%m-%dT%H:%M:%S%z')
+        self.request.session.modified = True
+
+        for er in self.registration.eventregistration_set.all():
+            firstName = form.cleaned_data.pop('er_%s_firstName' % er.id)
+            lastName = form.cleaned_data.pop('er_%s_lastName' % er.id)
+            email = form.cleaned_data.pop('er_%s_email' % er.id)
+            phone = form.cleaned_data.pop('er_%s_phone' % er.id, None)
+            student = form.cleaned_data.pop('er_%s_student' % er.id, False)
+
+            customer, created = Customer.objects.update_or_create(
+                first_name=firstName, last_name=lastName,
+                email=email, defaults={'phone': phone}
+            )
+            er.customer = customer
+            er.student = student
+            er.data.update(form.cleaned_data)
+            er.save()
+
+        # This signal allows vouchers to be applied temporarily, and it can
+        # be used for other tasks.  It is sent here because it has not been
+        # sent in the StudentInfoView if we got here.
+        post_student_info.send(
+            sender=StudentInfoView, invoice=self.invoice,
+            registration=self.registration
+        )
+        return HttpResponseRedirect(self.get_success_url())  # Redirect after POST
+
+
 class StudentInfoView(RegistrationAdjustmentsMixin, FormView):
     '''
     This page displays a preliminary total of what is being signed up for, and it also
@@ -1363,7 +1727,27 @@ class StudentInfoView(RegistrationAdjustmentsMixin, FormView):
             messages.info(request, _('Your registration session has expired. Please try again.'))
             return HttpResponseRedirect(reverse('registration'))
 
-        self.registration = Registration.objects.filter(invoice=self.invoice).first()
+        self.registration = Registration.objects.filter(
+            invoice=self.invoice
+        ).prefetch_related('eventregistration_set').first()
+
+        multiRegRule = getConstant('registration__multiRegNameFormRule')
+
+        self.multiReg = (
+            self.registration and
+            self.registration.eventregistration_set.count() > 1 and
+            (
+                multiRegRule == 'Y' or
+                (multiRegRule == 'O' and not self.registration.payAtDoor)
+            )
+        )
+
+        self.partnerRequired = (
+            self.registration and 
+            self.registration.eventregistration_set.filter(
+                event__partnerRequired=True
+            ).exists()
+        )
 
         # Ensure that session data is always updated when this view is called
         # so that passed voucher_ids are cleared.
@@ -1427,6 +1811,7 @@ class StudentInfoView(RegistrationAdjustmentsMixin, FormView):
             'addonItems': addons,
             'discount_codes': discount_codes,
             'discount_code_amount': total_discount_amount,
+            'is_multiple_registration': self.multiReg,
         })
 
         return context_data
@@ -1437,9 +1822,14 @@ class StudentInfoView(RegistrationAdjustmentsMixin, FormView):
         kwargs['request'] = self.request
         kwargs['registration'] = self.registration
         kwargs['invoice'] = self.invoice
+        kwargs['multiReg'] = self.multiReg
         return kwargs
 
     def get_success_url(self):
+        if self.multiReg:
+            return reverse('multiRegNameInfo')
+        elif self.partnerRequired:
+            return reverse('partnerRequiredForm')
         return reverse('showRegSummary')
 
     def form_valid(self, form):
@@ -1456,20 +1846,35 @@ class StudentInfoView(RegistrationAdjustmentsMixin, FormView):
             expiry.strftime('%Y-%m-%dT%H:%M:%S%z')
         self.request.session.modified = True
 
+        # Notice that student and phone are not popped so that they go into the
+        # Invoice data.
+        firstName = form.cleaned_data.pop('firstName')
+        lastName = form.cleaned_data.pop('lastName')
+        email = form.cleaned_data.pop('email')
+        phone = form.cleaned_data.get('phone', None)
+        student = form.cleaned_data.get('student', False)
+
         reg = self.registration
         if reg:
 
             # Update the expiration date for this registration, and pass in the data from
             # this form.
-            reg.firstName = form.cleaned_data.pop('firstName')
-            reg.lastName = form.cleaned_data.pop('lastName')
-            reg.email = form.cleaned_data.pop('email')
-            reg.phone = form.cleaned_data.pop('phone', None)
-            reg.student = form.cleaned_data.pop('student', False)
             reg.comments = form.cleaned_data.pop('comments', None)
             reg.howHeardAboutUs = form.cleaned_data.pop('howHeardAboutUs', None)
 
-            invoice = reg.link_invoice(expirationDate=expiry)
+            if not self.multiReg:
+                customer, created = Customer.objects.update_or_create(
+                    first_name=firstName, last_name=lastName,
+                    email=email, defaults={'phone': phone}
+                )
+                reg.eventregistration_set.update(
+                    customer=customer, student=student
+                )
+
+            invoice = reg.link_invoice(
+                expirationDate=expiry, firstName=firstName, lastName=lastName,
+                email=email, save=False
+            )
 
             # Anything else in the form goes to the Invoice data.
             invoice.data.update(form.cleaned_data)
@@ -1481,13 +1886,17 @@ class StudentInfoView(RegistrationAdjustmentsMixin, FormView):
             if invoice.status == Invoice.PaymentStatus.preliminary:
                 invoice.expirationDate = expiry
 
-            invoice.firstName = form.cleaned_data.pop('firstName')
-            invoice.lastName = form.cleaned_data.pop('lastName')
-            invoice.email = form.cleaned_data.pop('email')
+            invoice.firstName = firstName
+            invoice.lastName = lastName
+            invoice.email = email
             invoice.data.update(form.cleaned_data)
             invoice.save()
 
         # This signal allows vouchers to be applied temporarily, and it can
-        # be used for other tasks
-        post_student_info.send(sender=StudentInfoView, invoice=invoice, registration=reg)
+        # be used for other tasks.  We only send it here if we are not collecting
+        # additional name information in the next step.
+        if not self.multiReg:
+            post_student_info.send(
+                sender=StudentInfoView, invoice=invoice, registration=reg
+            )
         return HttpResponseRedirect(self.get_success_url())  # Redirect after POST
