@@ -435,6 +435,23 @@ class ShowDescriptionRule(ChoicePreference):
     default = 'all'
 
 
+@global_preferences_registry.register
+class RegistrationWidgetType(ChoicePreference):
+    section = registration
+    name = 'widgetType'
+    choices = [
+        ('AC', _('Checkboxes')),
+        ('AQ', _('Quantity Inputs')),
+        ('SC', _('Checkboxes for Class Series, Quantity Inputs for Other Events')),
+        ('SQ', _('Quantity Inputs for Class Series, Checkboxes for Other Events')),
+    ]
+    verbose_name = _('Input field type for registration page')
+    help_text = _(
+        'This widget is diplayed for each option on the registration page'
+    )
+    default = 'AQ'
+
+
 @global_preferences_registry.register
 class MultiRegSeriesRule(ChoicePreference):
     section = registration
@@ -500,6 +517,29 @@ class MultiRegDropInRule(ChoicePreference):
     default = 'Y'
 
 
+@global_preferences_registry.register
+class MultiRegNameFormRule(ChoicePreference):
+    section = registration
+    name = 'multiRegNameFormRule'
+    choices = [
+        ('Y', _('Always')),
+        ('O', _('Only registrations not at the door')),
+        ('N', _('Never')),
+    ]
+    verbose_name = _('Ask for names associated with each event registration')
+    help_text = _(
+        'When a customer submits more than one event registration at a time, ' +
+        'either for multiple events or multiple times for the same event, the ' +
+        'registration process can request that they confirm the name and ' +
+        'email associated with each individual event registration as an ' +
+        'additional step in the process. If this option is not selected, then ' +
+        'the customer information from Step 2 of the process will be assigned ' +
+        'to each event registration. A "direct" at-the-door registration that ' +
+        'skips Step 2 of the process will also skip this step.'
+    )
+    default = 'Y'
+
+
 @global_preferences_registry.register
 class DoorCheckInRule(ChoicePreference):
     section = registration
@@ -560,6 +600,15 @@ class AllowAjaxSignin(BooleanPreference):
     )
     default = True
 
+@global_preferences_registry.register
+class AddStudentField(BooleanPreference):
+    section = registration
+    name = 'addStudentField'
+    verbose_name = _('Include a "student" checkbox in the registration process')
+    help_text = _(
+        'If checked, then this checkbox can be used to offer student discounts.'
+    )
+    default = True
 
 @global_preferences_registry.register
 class RegistrationSessionExpiryMinutes(IntegerPreference):
